/*------------------------------------------------------------------------------
 * MDK Middleware - Component ::USB:Device
 * Copyright (c) 2004-2018 ARM Germany GmbH. All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    HID.c
 * Purpose: USB Device - Human Interface Device example
 *----------------------------------------------------------------------------*/

#include "cmsis_os.h"                   /* CMSIS RTOS definitions             */
#include "rl_usb.h"                     /* RL-USB function prototypes         */

#include "Board_LED.h"
#include "Board_Buttons.h"
#include "Board_GLCD.h"
#include "GLCD_Config.h"

#include "SCU_LPC18xx.h"

extern GLCD_FONT GLCD_Font_6x8;
extern GLCD_FONT GLCD_Font_16x24;

/*------------------------------------------------------------------------------
 *        Application
 *----------------------------------------------------------------------------*/
int main (void) {
  uint8_t but;
  uint8_t buf[2] = { 0, 0 };

  LED_Initialize    ();
  Buttons_Initialize();

  GLCD_Initialize         ();
  GLCD_SetBackgroundColor (GLCD_COLOR_BLUE);
  GLCD_SetForegroundColor (GLCD_COLOR_WHITE);
  GLCD_ClearScreen        ();
  GLCD_SetFont            (&GLCD_Font_16x24);
  GLCD_DrawString         (0, 0*24, "    USB Device      ");
  GLCD_DrawString         (0, 1*24, "    HID Class       ");
  GLCD_DrawString         (0, 2*24, "    HID Example     ");
  GLCD_DrawString         (0, 4*24, " USB0: HID0         ");
  GLCD_DrawString         (0, 5*24, " USB1: HID1         ");
  GLCD_DrawString         (0, 8*24, "  Keil Tools by ARM ");
  GLCD_DrawString         (0, 9*24, "    www.keil.com    ");

  /* Disable pull-ups on pins used for VBUS host drive signals P6_3 and P9_5  */
  SCU_PinConfigure (6, 3, SCU_CFG_MODE_FUNC0 | SCU_PIN_CFG_PULLUP_DIS);
  SCU_PinConfigure (9, 5, SCU_CFG_MODE_FUNC0 | SCU_PIN_CFG_PULLUP_DIS);

  USBD_Initialize         (0);          /* USB Device 0 Initialization        */
  USBD_Connect            (0);          /* USB Device 0 Connect               */

  USBD_Initialize         (1);          /* USB Device 1 Initialization        */
  USBD_Connect            (1);          /* USB Device 1 Connect               */

  while (1) {                           /* Loop forever                       */
    but = (uint8_t)(Buttons_GetState ());
    if ((but ^ buf[0]) & 3) {
      buf[0] = but & 3;
      USBD_HID_GetReportTrigger(0, 0, &buf[0], 1);
    }
    if (((but >> 2) ^ buf[1]) & 3) {
      buf[1] = (but >> 2) & 3;
      USBD_HID_GetReportTrigger(1, 0, &buf[1], 1);
    }
    osDelay(100);                       /* 100 ms delay for sampling buttons  */
  }
}
