/*------------------------------------------------------------------------------
 * MDK Middleware - Component ::USB:Device
 * Copyright (c) 2004-2018 ARM Germany GmbH. All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    VirtualCOM.c
 * Purpose: USB Device - Virtual COM Port example
 *----------------------------------------------------------------------------*/

#include "cmsis_os.h"                   /* CMSIS RTOS definitions             */
#include "rl_usb.h"                     /* RL-USB function prototypes         */

#include "Board_GLCD.h"
#include "GLCD_Config.h"

#include "SCU_LPC18xx.h"

extern GLCD_FONT GLCD_Font_6x8;
extern GLCD_FONT GLCD_Font_16x24;

/*------------------------------------------------------------------------------
 *        Application
 *----------------------------------------------------------------------------*/
int main (void) {

  GLCD_Initialize         ();
  GLCD_SetBackgroundColor (GLCD_COLOR_BLUE);
  GLCD_SetForegroundColor (GLCD_COLOR_WHITE);
  GLCD_ClearScreen        ();
  GLCD_SetFont            (&GLCD_Font_16x24);
  GLCD_DrawString         (0, 0*24, "    USB Device      ");
  GLCD_DrawString         (0, 1*24, "   CDC ACM Class    ");
  GLCD_DrawString         (0, 2*24, " VirtualCOM Example ");
  GLCD_DrawString         (0, 4*24, " USB_0 <-> UART0    ");
  GLCD_DrawString         (0, 5*24, " USB_1 <-> UART3    ");
  GLCD_DrawString         (0, 8*24, "  Keil Tools by ARM ");
  GLCD_DrawString         (0, 9*24, "    www.keil.com    ");
  GLCD_SetFont            (&GLCD_Font_6x8);
  GLCD_DrawString         (0, 21*8, "Note: For UART0 first power-up the board and then       ");
  GLCD_DrawString         (0, 22*8, "      connect UART cable to UART0 port on the board     ");

  /* Disable pull-ups on pins used for VBUS host drive signals P6_3 and P9_5  */
  SCU_PinConfigure (6, 3, SCU_CFG_MODE_FUNC0 | SCU_PIN_CFG_PULLUP_DIS);
  SCU_PinConfigure (9, 5, SCU_CFG_MODE_FUNC0 | SCU_PIN_CFG_PULLUP_DIS);

  USBD_Initialize         (0);          /* USB Device 0 Initialization        */
  USBD_Connect            (0);          /* USB Device 0 Connect               */

  USBD_Initialize         (1);          /* USB Device 1 Initialization        */
  USBD_Connect            (1);          /* USB Device 1 Connect               */

  while (1) {                           /* Loop forever                       */
    osSignalWait (0U, osWaitForever);
  }
}
